#include "FreeRTOS.h"
#include "task.h"
#include "queue.h"
#include "lcd.h"
#include "defs.h"

#define lcdSHORT_DELAY	1

#define LCD_D3 PORTCbits.RC7	// Data bits
#define LCD_D2 PORTCbits.RC6	// Data bits
#define LCD_D1 PORTCbits.RC5	// Data bits
#define LCD_D0 PORTCbits.RC4	// Data bits

#define DISP_2Line_4Bit		0b00101000
#define	DISP_ON				0b00001100
#define	ENTRY_INC			0b00000001
#define	CLR_DISP			0b00000110

#define	CLR_LCD_PORT		0xff0f
#define	FUNC_SET_H			0x0080
#define	FUNC_SET_L			0x0030

void prvLCDPutString( portCHAR *pcString )
{
	/* Write out each character with appropriate delay between each. */
	while(*pcString)                         // Write data to LCD up to null
    {    
        XLCDPut(*pcString);                   // Write character to LCD
        pcString++;                           // Increment buffer
    }
    return;	
}
/*-----------------------------------------------------------*/
	
void XLCDCommand(unsigned char cmd)
{	
	static unsigned portBASE_TYPE xCmd = 0;
	
	xCmd = cmd;
	
    vTaskDelay( lcdSHORT_DELAY );
	XLCD_RSPIN =0;
	Nop();
	XLCD_ENPIN =0;
	Nop();

	if(xCmd & 0x80) LATC |= 128; else LATC &= ~128;
	if(xCmd & 0x40) LATC |= 64; else LATC &= ~64;
	if(xCmd & 0x20) LATC |= 32; else LATC &= ~32;
	if(xCmd & 0x10) LATC |= 16; else LATC &= ~16;

    XLCD_ENPIN = 1;   
    Nop();  
    Nop();
    Nop();
    Nop();
    XLCD_ENPIN = 0;
    
    xCmd = cmd;

	if(xCmd & 0x08) LATC |= 128; else LATC &= ~128;
	if(xCmd & 0x04) LATC |= 64; else LATC &= ~64;
	if(xCmd & 0x02) LATC |= 32; else LATC &= ~32;
	if(xCmd & 0x01) LATC |= 16; else LATC &= ~16;
	 
    XLCD_ENPIN = 1;   
    Nop();  
    Nop();
    Nop();
    Nop();
    XLCD_ENPIN = 0;
    
    vTaskDelay( lcdSHORT_DELAY );
    return;
}

void XLCDPut(char data)
{
	static portBASE_TYPE xData;
	
	xData = data;
	
    vTaskDelay( lcdSHORT_DELAY );
	XLCD_RSPIN =1;
	Nop();
	XLCD_ENPIN =0;
	Nop();
	
	if(xData & 0x80) LATC |= 128; else LATC &= ~128;
	if(xData & 0x40) LATC |= 64; else LATC &= ~64;
	if(xData & 0x20) LATC |= 32; else LATC &= ~32;
	if(xData & 0x10) LATC |= 16; else LATC &= ~16;
        
    XLCD_ENPIN = 1;   
    Nop();  
    Nop();
    Nop();
    Nop();
    XLCD_ENPIN = 0;
    
    xData = data;

  	if(xData & 0x08) LATC |= 128; else LATC &= ~128;
	if(xData & 0x04) LATC |= 64; else LATC &= ~64;
	if(xData & 0x02) LATC |= 32; else LATC &= ~32;
	if(xData & 0x01) LATC |= 16; else LATC &= ~16;
        
    XLCD_ENPIN = 1;   
    Nop();  
    Nop();
    Nop();
    Nop();
    XLCD_ENPIN = 0;
    
    vTaskDelay( lcdSHORT_DELAY );
   return;
}

/*-----------------------------------------------------------*/

void prvSetupLCD( void )
{
	vTaskDelay( lcdSHORT_DELAY );
	
	XLCD_DATAPORT_TRIS  &= CLR_LCD_PORT;
    XLCD_DATAPORT &= CLR_LCD_PORT;

	XLCD_RSPIN_TRIS =0;                         //make control ports output
	XLCD_ENPIN_TRIS =0;
	
	vTaskDelay( lcdSHORT_DELAY );
	vTaskDelay( lcdSHORT_DELAY );
	
	XLCD_DATAPORT &= CLR_LCD_PORT;    // Clear lower port*********************
    XLCD_DATAPORT |= FUNC_SET_L; // Function set cmd(4-bit interface)

    XLCD_ENPIN = 1;                 // Clock the cmd in
    vTaskDelay( lcdSHORT_DELAY );
    XLCD_ENPIN = 0;
    
    vTaskDelay( lcdSHORT_DELAY );
    
    XLCD_DATAPORT   &= CLR_LCD_PORT;    // Clear lower port
    XLCD_DATAPORT   |= FUNC_SET_L;  // Function set cmd(4-bit interface)

	vTaskDelay( lcdSHORT_DELAY );

    XLCD_ENPIN = 1;                 // Clock the cmd in
    vTaskDelay( lcdSHORT_DELAY );  
    XLCD_ENPIN = 0;

    vTaskDelay( lcdSHORT_DELAY );
    
    XLCD_DATAPORT   &= CLR_LCD_PORT;    // Clear lower port
    XLCD_DATAPORT   |= FUNC_SET_H;  // Function set cmd(4-bit interface)

	vTaskDelay( lcdSHORT_DELAY ); 
	
    XLCD_ENPIN = 1;                 // Clock the cmd in
    vTaskDelay( lcdSHORT_DELAY );   
    XLCD_ENPIN = 0;

    XLCDCommand( DISP_2Line_4Bit );    //if 2Line 5x8
	vTaskDelay( lcdSHORT_DELAY ); 
 
    XLCDCommand( DISP_ON );        //display clear
   	vTaskDelay( lcdSHORT_DELAY ); 
    
    XLCDCommand( ENTRY_INC );    //if cursor inc and display shift
    vTaskDelay( lcdSHORT_DELAY ); 
    
	XLCDCommand( CLR_DISP );
    vTaskDelay( lcdSHORT_DELAY );
}
/*-----------------------------------------------------------*/


void prvLCDClear( void )
{
	//Clear the display. 
	XLCDCommand(lcdCLEAR_DISPLAY);
	vTaskDelay( lcdSHORT_DELAY );	
}
		
	
	




